@@ -1,6 +1,6 @@
 /*
  *  Copyright 2008-2013 NVIDIA Corporation
- *  Modifications Copyright© 2019 Advanced Micro Devices, Inc. All rights reserved.
+ *  Modifications Copyright© 2019-2023 Advanced Micro Devices, Inc. All rights reserved.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -743,8 +743,6 @@ TEST(ScanTests, TestExclusiveScanDevice)
     }
 }
 
-#if THRUST_CPP_DIALECT >= 2011
-
 struct Int {
     int i{};
     __host__ __device__ explicit Int(int num) : i(num) {}
@@ -764,5 +762,3 @@ TEST(ScanTests, TestInclusiveScanWithUserDefinedType)
 
     ASSERT_EQ(static_cast<Int>(vec.back()).i, 5);
 }
-
-#endif // c++11
